@@ -1,9 +1,12 @@
 package RDF::LinkedData;
 
-use Moose;
+use strict;
+use warnings;
+use Moo;
 use namespace::autoclean;
+use Types::Standard qw(InstanceOf Str Bool Maybe Int HashRef);
 
-use RDF::Trine qw[iri literal blank statement];
+use RDF::Trine qw[iri literal blank statement variable];
 use RDF::Trine::Serializer;
 use RDF::Trine::Namespace;
 use Log::Log4perl qw(:easy);
@@ -13,15 +16,17 @@ use URI::NamespaceMap;
 use URI;
 use HTTP::Headers;
 use Module::Load::Conditional qw[can_load];
-use MooseX::UndefTolerant::Attribute;
 use Encode;
 use RDF::RDFa::Generator 0.102;
 use HTML::HTML5::Writer qw(DOCTYPE_XHTML_RDFA);
 use Data::Dumper;
-use Digest::MD5 ('md5_hex');
-
-
-with 'MooseX::Log::Log4perl::Easy';
+use Digest::MD5 ('md5_base64');
+use Carp;
+use Try::Tiny;
+use List::Util qw(any);
+use Log::Log4perl;
+use Log::Log4perl ':easy';
+use Log::Contextual qw( :log ), -package_logger => Log::Log4perl->get_logger;
 
 BEGIN {
 	if ($ENV{TEST_VERBOSE}) {
@@ -33,6 +38,7 @@ BEGIN {
 											 category => 'RDF.LinkedData' 
 										  } );
 	}
+	use Log::Contextual -logger => Log::Log4perl->get_logger;
 }
 
 
@@ -40,23 +46,24 @@ BEGIN {
 
 =head1 NAME
 
-RDF::LinkedData - A simple Linked Data server implementation
+RDF::LinkedData - A Linked Data server implementation
 
 =head1 VERSION
 
-Version 0.58
+Version 0.74
 
 =cut
 
- our $VERSION = '0.58';
+ our $VERSION = '0.74';
 
 
 =head1 SYNOPSIS
 
 For just setting this up and get it to run, you would just use the
-C<linked_data.psgi> script in this distribution. The usage of that is documented in
-L<Plack::App::RDF::LinkedData>. If you want to try and use this
-directly, you'd do stuff like:
+C<linked_data.psgi> script in this distribution. The usage of that is
+documented in L<Plack::App::RDF::LinkedData>, with the README is a
+quick start guide. If you want to try and use this directly, you'd do
+stuff like:
 
 	my $ld = RDF::LinkedData->new(store => $config->{store},
                                  endpoint_config => $config->{endpoint},
@@ -70,12 +77,18 @@ See L<Plack::App::RDF::LinkedData> for a complete example.
 
 =head1 DESCRIPTION
 
-This module is used to create a minimal Linked Data server that can
+This module is used to create a Linked Data server that can
 serve RDF data out of an L<RDF::Trine::Model>. It will look up URIs in
 the model and do the right thing (known as the 303 dance) and mint
 URLs for that, as well as content negotiation. Thus, you can
 concentrate on URIs for your things, you need not be concerned about
-minting URLs for the pages to serve it.
+minting URLs for the pages to serve it. In addition, optional modules
+can provide other important functionalities: Cross-origin resource
+sharing, VoID description, cache headers, SPARQL Endpoint, Triple
+Pattern Fragments, etc. As such, it encompasses a fair share of
+Semantic Web best practices, but possibly not in a very flexible Big
+Data manner.
+
 
 =head1 METHODS
 
@@ -94,21 +107,21 @@ L<Plack::Request> object (must be passed before you call C<content>)
 and an C<endpoint_config> hashref if you want to have a SPARQL
 Endpoint running using the recommended module L<RDF::Endpoint>.
 
-This module can also provide additional triples to turn the respons
+This module can also provide additional triples to turn the response
 into a hypermedia type. If you don't want this, set the C<hypermedia>
 argument to false. Currently this entails setting the SPARQL endpoint
 and vocabularies used using the L<VoID vocabulary|http://vocab.deri.ie/void>.
-The latter is very limited at present, all it'll do is use the namespaces
-if you have C<namespaces_as_vocabularies> enabled, which it is by default.
+
+Finally, it can provide experimental L<Triple Pattern
+Fragments|http://www.hydra-cg.com/spec/latest/triple-pattern-fragments/>
+support.
 
 =item C<< BUILD >>
 
-Called by Moose to initialize an object.
+Called by Moo to initialize an object.
 
 =cut
 
-# TODO look at buildargs to replace UndefTolerant
-
 sub BUILD {
 	my $self = shift;
 
@@ -118,7 +131,7 @@ sub BUILD {
 	}
 
  	if ($self->has_endpoint_config) {
-		$self->logger->debug('Endpoint config found with parameters: ' . Dumper($self->endpoint_config) );
+		log_debug {'Endpoint config found with parameters: ' . Dumper($self->endpoint_config) };
 
 		unless (can_load( modules => { 'RDF::Endpoint' => 0.03 })) {
 			throw Error -text => "RDF::Endpoint not installed. Please install or remove its configuration.";
@@ -130,11 +143,11 @@ sub BUILD {
 
 		$self->endpoint(RDF::Endpoint->new($self->model, $self->endpoint_config));
  	} else {
-		$self->logger->info('No endpoint config found');
+		log_info {'No endpoint config found'};
 	}
 
  	if ($self->has_void_config) {
-		$self->logger->debug('VoID config found with parameters: ' . Dumper($self->void_config) );
+		log_debug {'VoID config found with parameters: ' . Dumper($self->void_config) };
 
 		unless (can_load( modules => { 'RDF::Generator::Void' => 0.04 })) {
 			throw Error -text => "RDF::Generator::Void not installed. Please install or remove its configuration.";
@@ -146,12 +159,31 @@ sub BUILD {
 		$self->void(RDF::Generator::Void->new(inmodel => $self->model, 
 														  dataset_uri => $dataset_uri,
 														  namespaces_as_vocabularies => $self->void_config->{namespaces_as_vocabularies}));
+		if ($self->has_fragments) {
+			log_debug {'Triple Pattern Fragments config found with parameters: ' . Dumper($self->fragments_config) };
+		}
  	} else {
-		$self->logger->info('No VoID config found');
+		log_info {'No VoID config found'};
 	}
 }
 
-has store => (is => 'rw', isa => 'HashRef | Str' );
+=item C<< BUILDARGS >>
+
+Called by Moo to ensure that some attributes can be left unset.
+
+=cut
+
+around BUILDARGS => sub 
+  {
+	  my ($next, $self) = (shift, shift);
+	  my $args = $self->$next(@_);
+	  for (keys %$args) {
+		  delete $args->{$_} if not defined $args->{$_};
+	  }
+	  return $args;
+  };
+
+has store => (is => 'rw', isa => HashRef | Str );
 
 
 =item C<< model >>
@@ -160,7 +192,7 @@ Returns the RDF::Trine::Model object.
 
 =cut
 
-has model => (is => 'ro', isa => 'RDF::Trine::Model', lazy => 1, builder => '_build_model', 
+has model => (is => 'ro', isa => InstanceOf['RDF::Trine::Model'], lazy => 1, builder => '_build_model', 
 				  handles => { current_etag => 'etag' });
 
 sub _build_model {
@@ -191,17 +223,17 @@ Returns or sets the base URI for this handler.
 
 =cut
 
-has base_uri => (is => 'rw', isa => 'Str', default => '' );
+has base_uri => (is => 'rw', isa => Str, default => '' );
 
-has hypermedia => (is => 'ro', isa => 'Bool', default => 1);
+has hypermedia => (is => 'ro', isa => Bool, default => 1);
 
-has namespaces_as_vocabularies => (is => 'ro', isa => 'Bool', default => 1);
+has namespaces_as_vocabularies => (is => 'ro', isa => Bool, default => 1);
 
-has endpoint_config => (is => 'rw', traits => [ qw(MooseX::UndefTolerant::Attribute)],
-								isa=>'HashRef', predicate => 'has_endpoint_config');
+has endpoint_config => (is => 'rw', isa=>Maybe[HashRef], predicate => 'has_endpoint_config');
 
-has void_config => (is => 'rw', traits => [ qw(MooseX::UndefTolerant::Attribute)],
-								isa=>'HashRef', predicate => 'has_void_config');
+has void_config => (is => 'rw', isa=>Maybe[HashRef], predicate => 'has_void_config');
+
+has fragments_config => (is => 'rw', isa=>Maybe[HashRef], predicate => 'has_fragments');
 
 
 
@@ -211,7 +243,7 @@ Returns the L<Plack::Request> object if it exists or sets it if a L<Plack::Reque
 
 =cut
 
-has request => ( is => 'rw', isa => 'Plack::Request');
+has request => ( is => 'rw', isa => InstanceOf['Plack::Request']);
 
 
 =item C<< current_etag >>
@@ -224,7 +256,7 @@ Returns or sets the last Etag of so that changes to the model can be detected.
 
 =cut
 
-has last_etag => ( is => 'rw', isa => 'Str', predicate => 'has_last_etag');
+has last_etag => ( is => 'rw', isa => Str, predicate => 'has_last_etag');
 
 
 =item namespaces ( { skos => 'http://www.w3.org/2004/02/skos/core#', dct => 'http://purl.org/dc/terms/' } )
@@ -234,7 +266,7 @@ Gets or sets the namespaces that some serializers use for pretty-printing.
 =cut
 
 has 'namespaces' => (is => 'rw', 
-							isa => 'URI::NamespaceMap',
+							isa => InstanceOf['URI::NamespaceMap'],
 							builder => '_build_namespaces',
 							lazy => 1,
 							handles => {
@@ -275,6 +307,10 @@ sub response {
 
 	my $headers_in = $self->request->headers;
 
+	my $server = "RDF::LinkedData/$VERSION";
+	$server .= " " . $response->headers->header('Server') if defined($response->headers->header('Server'));
+	$response->headers->header('Server' => $server);
+
 	my $endpoint_path;
 	if ($self->has_endpoint) {
 	  $endpoint_path = $self->endpoint_config->{endpoint_path};
@@ -283,6 +319,96 @@ sub response {
 	  }
 	}
 
+	if ($self->has_fragments && ($uri->path eq $self->fragments_config->{fragments_path})) {
+		croak 'A VoID description is needed when using Triple Pattern Fragments' unless ($self->has_void);
+
+		# First compute the selectors from the query parameters
+		my %params = $uri->query_form;
+		my %statement = (subject => undef,
+							  predicate => undef,
+							  object => undef);
+		foreach my $term (keys(%statement)) {
+			my $value = $params{$term};
+			next unless $value;
+			return _client_error($response, "$term is invalid") if ref($value); # E.g. an array would be invalid
+			if ($value =~ m/^\?(\S+)$/) { # Regexp matching variable
+				$statement{$term} = variable($1);
+			} elsif (($term eq 'object') && ($value =~ m/^\"(.+)\"((\@|\^\^)(\S+))?$/)) { # regexp matching literal
+				my $string = $1;
+				my $lang_or_datatype = $3;
+				my $rest = $4;
+				if (defined($lang_or_datatype) && ($lang_or_datatype eq '@')) {
+					$statement{$term} = literal($string, $rest);
+				} else {
+					$statement{$term} = literal($string, undef, $rest);
+				}
+			} else { # Now, it may be an IRI
+				try {
+					$statement{$term} = iri($value);
+				} catch {
+					return _client_error($response, 'Was not able to parse subject as a IRI');
+				}
+			}
+		}
+
+		log_debug {'Getting fragment with this selector ' . Dumper(\%statement) };
+		return _client_error($response, 'Returning the whole database not allowed') 
+				unless $self->fragments_config->{allow_dump_dataset} || any { defined } values(%statement);
+		my $output_model = $self->_common_fragments_control;
+
+		my $iterator = $self->model->get_statements($statement{subject}, $statement{predicate}, $statement{object});
+		$output_model->begin_bulk_ops;
+		my $counter = 0;
+		while (my $st = $iterator->next) {
+			$counter++;
+			# TODO: Paging goes here
+			$output_model->add_statement($st);
+		}
+		$self->add_namespace_mapping(void => 'http://rdfs.org/ns/void#');
+		$self->add_namespace_mapping(hydra => 'http://www.w3.org/ns/hydra/core#');
+		my $cl = literal($counter, undef, 'http://www.w3.org/2001/XMLSchema#integer');
+		my $void = RDF::Trine::Namespace->new('http://rdfs.org/ns/void#');
+		$output_model->add_statement(statement(iri($uri), 
+															$void->triples,
+															$cl));
+		$output_model->add_statement(statement(iri($uri), 
+															iri('http://www.w3.org/ns/hydra/core#totalItems'),
+															$cl));
+		$output_model->add_statement(statement(iri($uri),
+															iri('http://www.w3.org/1999/02/22-rdf-syntax-ns#type'),
+															$void->Dataset));
+		$output_model->add_statement(statement($self->void->dataset_uri,
+															$void->subset,
+															iri($uri)));
+		$output_model->add_statement(statement(iri($uri),
+															iri('http://purl.org/dc/terms/source'),
+															$self->void->dataset_uri
+															));
+		$output_model->end_bulk_ops;
+		my ($ct, $s);
+		try {
+			($ct, $s) = RDF::Trine::Serializer->negotiate('request_headers' => $headers_in,
+																		 base_uri => $self->base_uri,
+																		 namespaces => $self->_namespace_hashref);
+		} catch {
+			$response->status(406);
+			$response->headers->content_type('text/plain');
+			$response->body('HTTP 406: No serialization available any specified content type');
+			return $response;
+		};
+
+		$response->status(200);
+		$response->headers->header('Vary' => join(", ", qw(Accept)));
+		if ($self->has_last_etag) {
+			$response->headers->header('ETag' => '"' . md5_base64($self->last_etag . $ct) . '"');
+		}
+		my $body = $s->serialize_model_to_string($output_model);
+		log_trace { "Fragment message body is $body" };
+		$response->headers->content_type($ct);
+		$response->body(encode_utf8($body));
+		return $response;
+	}
+
 	if ($self->has_void) {
 		my $void_resp = $self->_void_content($uri, $endpoint_path);
 		return $void_resp if (defined($void_resp));
@@ -291,7 +417,7 @@ sub response {
 	my $type = $self->type;
 	$self->type('');
 	my $node = $self->my_node($uri);
-	$self->logger->info("Try rendering '$type' page for subject node: " . $node->as_string);
+	log_info{"Try rendering '$type' page for subject node: " . $node->as_string};
 	if ($self->count($node) > 0) {
 		if ($type) {
 			my $preds = $self->helper_properties;
@@ -303,24 +429,22 @@ sub response {
 				return $response;
 			}
 
-			$self->logger->debug("Will render '$type' page ");
+			log_debug {"Will render '$type' page " };
 			if ($headers_in->can('header') && $headers_in->header('Accept')) {
-				$self->logger->debug('Found Accept header: ' . $headers_in->header('Accept'));
+				log_debug {'Found Accept header: ' . $headers_in->header('Accept') };
 			} else {
 				$headers_in->header('Accept' => 'application/rdf+xml');
 				if ($headers_in->header('Accept')) {
-					$self->logger->warn('Setting Accept header: ' . $headers_in->header('Accept'));
+					log_warn { 'Setting Accept header: ' . $headers_in->header('Accept') };
 				} else {
-					$self->logger->warn('No content type header can be set');
+					log_warn { 'No content type header can be set' };
 				}
 			}
 			$response->status(200);
 			my $content = $self->_content($node, $type, $endpoint_path);
 			$response->headers->header('Vary' => join(", ", qw(Accept)));
 			if (defined($self->current_etag)) {
-				$self->current_etag =~ m|(^W/)|; # If the ETag is declared as weak, preserve that
-				my $weak = defined($1) ? $1 : '';
-				$response->headers->header('ETag' => $weak . md5_hex($self->current_etag . $content->{content_type}));
+			  $response->headers->header('ETag' => '"' . md5_base64($self->current_etag . $content->{content_type}) . '"');
 			}
 			$response->headers->content_type($content->{content_type});
 			$response->body(encode_utf8($content->{body}));
@@ -333,7 +457,7 @@ sub response {
 				my $preds = $self->helper_properties;
 				$newurl = $preds->page($node);
 			}
-			$self->logger->debug('Will do a 303 redirect to ' . $newurl);
+			log_debug {'Will do a 303 redirect to ' . $newurl };
 			$response->headers->header('Location' => $newurl);
 			$response->headers->header('Vary' => join(", ", qw(Accept)));
 		}
@@ -351,6 +475,15 @@ sub response {
 	return $response;
 }
 
+sub _client_error {
+	my ($response, $msg) = @_;
+	$response->status(400);
+	$response->headers->content_type('text/plain');
+	$response->body("HTTP 400: $msg");
+	return $response;
+}
+
+
 
 =item C<< helper_properties (  ) >>
 
@@ -359,7 +492,7 @@ it if a L<RDF::Helper::Properties> object is given as parameter.
 
 =cut
 
-has helper_properties => ( is => 'rw', isa => 'RDF::Helper::Properties', lazy => 1, builder => '_build_helper_properties');
+has helper_properties => ( is => 'rw', isa => InstanceOf['RDF::Helper::Properties'], lazy => 1, builder => '_build_helper_properties');
 
 sub _build_helper_properties {
 	my $self = shift;
@@ -374,7 +507,7 @@ Returns or sets the type of result to return, i.e. C<page>, in the case of a hum
 
 =cut
 
-has 'type' => (is => 'rw', isa => 'Str', default => ''); 
+has 'type' => (is => 'rw', isa => Str, default => ''); 
 
 
 =item C<< my_node >>
@@ -387,10 +520,7 @@ get a URI object containing the full URI of the node.
 
 sub my_node {
 	my ($self, $iri) = @_;
-    
-	# not happy with this, but it helps for clients that do content sniffing based on filename
-	$iri =~ s/.(nt|rdf|ttl)$//;
-	$self->logger->info("Subject URI to be used: $iri");
+	log_info { "Subject URI to be used: $iri" };
 	return RDF::Trine::Node::Resource->new( $iri );
 }
 
@@ -461,7 +591,7 @@ sub _content {
 			$iter = $iter->concat($hmmodel->as_stream);
 		}
 		$output{body} = $s->serialize_iterator_to_string ( $iter );
-		$self->logger->trace("Message body is $output{body}");
+		log_trace { "Message body is $output{body}" };
 
 	} else {
 		$self->{_type} = 'page';
@@ -493,7 +623,7 @@ constructor, so you would most likely not use this method.
 =cut
 
 
-has endpoint => (is => 'rw', isa => 'RDF::Endpoint', predicate => 'has_endpoint');
+has endpoint => (is => 'rw', isa => InstanceOf['RDF::Endpoint'], predicate => 'has_endpoint');
 
 
 =item C<< void ( [ $voidg ] ) >>
@@ -507,7 +637,7 @@ method.
 =cut
 
 
-has void => (is => 'rw', isa => 'RDF::Generator::Void', predicate => 'has_void');
+has void => (is => 'rw', isa => InstanceOf['RDF::Generator::Void'], predicate => 'has_void');
 
 
 sub _negotiate {
@@ -521,16 +651,16 @@ sub _negotiate {
 																					'text/html' => 'html',
 																					'application/xhtml+xml' => 'xhtml'
 																				  }
-																	)
-	};
-	$self->logger->debug("Got $ct content type");
-	if ($@) {
+																	);
+		log_debug { "Got $ct content type" };
+		1;
+	} or do {
 		my $response = Plack::Response->new;
 		$response->status(406);
 		$response->headers->content_type('text/plain');
 		$response->body('HTTP 406: No serialization available any specified content type');
 		return $response;
-	}
+	};
 	return ($ct, $s)
 }
 
@@ -574,7 +704,9 @@ sub _void_content {
 				$generator->urispace($self->base_uri);
 			}
 			if ($self->namespaces_as_vocabularies) {
-				$generator->add_vocabularies($self->list_namespaces);
+			  foreach my $nsuri ($self->list_namespaces) {
+				 $generator->add_vocabularies($nsuri->as_string); # TODO: Should be fixed in RDF::Generator::Void, but we fix it here for now
+			  }
 			}
 			if ($self->has_endpoint) {
 				$generator->add_endpoints($self->base_uri . $endpoint_path);
@@ -597,6 +729,11 @@ sub _void_content {
 			$self->last_etag($self->current_etag);
 		}
 
+		if ($self->has_fragments) {
+			$self->add_namespace_mapping(hydra => 'http://www.w3.org/ns/hydra/core#');
+			$self->_common_fragments_control($self->_voidmodel);
+		}
+
 		# Now start serializing.
 		my ($ct, $s) = $self->_negotiate($self->request->headers);
 		return $ct if ($ct->isa('Plack::Response')); # A hack to allow for the failed conneg case
@@ -620,9 +757,7 @@ sub _void_content {
 		$etag = $self->_last_extvoid_mtime if ($self->void_config->{add_void});
 		$etag .= $self->last_etag if (defined($self->last_etag));
 		if ($etag) {
-			$etag =~ m|(^W/)|; # If the ETag is declared as weak, preserve that
-			my $weak = defined($1) ? $1 : '';
-			$response->headers->header('ETag' => $weak . md5_hex($etag . $ct));
+		  $response->headers->header('ETag' => '"' . md5_base64($etag . $ct) . '"');
 		}
 		$response->headers->content_type($ct);
 		$response->body(encode_utf8($body));
@@ -632,12 +767,72 @@ sub _void_content {
 	}
 }
 
-has _voidmodel => (is => 'rw', isa => 'RDF::Trine::Model', predicate => '_has_voidmodel', clearer => '_clear_voidmodel');
+has _voidmodel => (is => 'rw', isa => InstanceOf['RDF::Trine::Model'], predicate => '_has_voidmodel', clearer => '_clear_voidmodel');
 
-has _current_extvoid_mtime => (is => 'rw', isa => 'Int');
+has _current_extvoid_mtime => (is => 'rw', isa => Int);
 
-has _last_extvoid_mtime => (is => 'rw', isa => 'Int');
+has _last_extvoid_mtime => (is => 'rw', isa => Int);
 
+sub _common_fragments_control {
+	my $self = shift;
+	my $model = shift || RDF::Trine::Model->temporary_model;
+	my $void = RDF::Trine::Namespace->new('http://rdfs.org/ns/void#');
+	my $xsd  = RDF::Trine::Namespace->new('http://www.w3.org/2001/XMLSchema#');
+	my $hydra = RDF::Trine::Namespace->new('http://www.w3.org/ns/hydra/core#');
+	my $rdf = RDF::Trine::Namespace->new('http://www.w3.org/1999/02/22-rdf-syntax-ns#');
+	$model->begin_bulk_ops;
+	my $void_subject = $self->void->dataset_uri;
+	$model->add_statement(statement($void_subject,
+											  $rdf->type,
+											  $hydra->Collection));
+	$model->add_statement(statement($void_subject,
+											  $rdf->type,
+											  $void->Dataset));
+	$model->add_statement(statement($void_subject,
+											  $hydra->search,
+											  blank('template')));
+	$model->add_statement(statement($void_subject,
+								 $void->uriLookupEndpoint,
+								 literal($self->base_uri . $self->fragments_config->{fragments_path}
+											. '{?subject,predicate,object}')));
+	$model->add_statement(statement(blank('template'),
+								 $hydra->template,
+								 literal($self->base_uri . $self->fragments_config->{fragments_path}
+											. '{?subject,predicate,object}')));
+	
+	$model->add_statement(statement(blank('template'),
+								  $hydra->mapping,
+								  blank('subject')));
+	$model->add_statement(statement(blank('template'),
+								  $hydra->mapping,
+								  blank('predicate')));
+	$model->add_statement(statement(blank('template'),
+								  $hydra->mapping,
+								  blank('object')));
+
+	$model->add_statement(statement(blank('subject'),
+								 $hydra->property,
+								 $rdf->subject));
+	$model->add_statement(statement(blank('subject'),
+								 $hydra->variable,
+								 literal('subject')));
+
+	$model->add_statement(statement(blank('predicate'),
+								 $hydra->property,
+								 $rdf->predicate));
+	$model->add_statement(statement(blank('predicate'),
+								 $hydra->variable,
+								 literal('predicate')));
+
+	$model->add_statement(statement(blank('object'),
+								 $hydra->property,
+								 $rdf->object));
+	$model->add_statement(statement(blank('object'),
+								 $hydra->variable,
+								 literal('object')));
+	$model->end_bulk_ops;
+	return $model;
+}
 
 =back
 
@@ -646,6 +841,10 @@ has _last_extvoid_mtime => (is => 'rw', isa => 'Int');
 
 Kjetil Kjernsmo, C<< <kjetilk@cpan.org> >>
 
+=head1 CONTRIBUTORS
+
+Toby Inkster
+
 =head1 BUGS
 
 Please report any bugs using L<github|https://github.com/kjetilk/RDF-LinkedData/issues>
@@ -671,8 +870,6 @@ L<http://lists.perlrdf.org/listinfo/dev>
 
 =item * Make it read-write hypermedia.
 
-=item * Use a environment variable for config on the command line?
-
 =item * Make the result graph configurable.
 
 =back
@@ -690,7 +887,7 @@ Copyright 2010 Gregory Todd Williams
 
 Copyright 2010 ABC Startsiden AS
 
-Copyright 2010, 2011, 2012, 2013 Kjetil Kjernsmo
+Copyright 2010, 2011, 2012, 2013, 2014, 2015 Kjetil Kjernsmo
 
 This program is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
@@ -698,7 +895,4 @@ under the same terms as Perl itself.
 
 =cut
 
-# TODO : immutable doesn't seem to work with UndefTolerant
-#__PACKAGE__->meta->make_immutable();
-
 1;
